//
// C++ Interface: cActionBase
//
// Description: action base
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef CACTIONBASE_H
#define CACTIONBASE_H

#include <qstring.h>

struct cActionBasePrivate;

class cTextChunk;
enum ParamType { PT_INT, PT_STRING, PT_NOTHING, PT_TEXTCHUNK };

/**
Base class for every object that wants to use cActionManager.

@author Tomas Mecir
*/

class cActionBase {
public:
  /** Constructor. Registers itself with cActionManager, if session >= 0 */
  cActionBase (const QString &name, int session = -1);
  /** destructor */
  virtual ~cActionBase ();

  /** object's name */
  QString objName () { return _name; };
  /** session number */
  int sess ();
  
  int attrib (const QString &name);
  void setAttrib (const QString &name, int value);
  QString strAttrib (const QString &name);
  void setStrAttrib (const QString &name, const QString &value);
protected:
  void addEventHandler (QString name, int priority, ParamType pt);
  void removeEventHandler (QString name);
  void addGlobalEventHandler (QString name, int priority, ParamType pt);
  void removeGlobalEventHandler (QString name);

  cActionBase *object (const QString &name, int session = -1);

// the following 8 methods exist, so that we don't have to #include cActionManager everywhere
  /** call an object's action - PT_INT parameter */
  QString callAction (QString objectName, QString action, int session, int par1, int par2=0);
  /** call an object's action - PT_STRING parameter */
  QString callAction (QString objectName, QString action, int session,
      QString &par1, const QString &par2=QString::null);
  /** as above, but with const string */
  QString callAction (QString objectName, QString action, int session,
      const QString &par1, const QString &par2=QString::null);
  /** call an object's action - PT_NOTHING parameter */
  QString callAction (QString objectName, QString action, int session);
  /** call an object's action - PT_TEXTCHUNK parameter */
  QString callAction (QString objectName, QString action, int session, cTextChunk *par);
  /** invoke an event - PT_INT parameter */
  void invokeEvent (QString event, int session, int par1, int par2 = 0);
  /** invoke an event - PT_STRING parameter */
  void invokeEvent (QString event, int session, QString &par1,
      const QString &par2 = QString::null);
  /** as above, but const param - so that passing constant strings works */
  void invokeEvent (QString event, int session, const QString &par1,
      const QString &par2 = QString::null);
  /** invoke an event - PT_NOTHING parameter */
  void invokeEvent (QString event, int session);
  /** invoke an event - PT_TEXTCHUNK parameter */
  void invokeEvent (QString event, int session, cTextChunk *par);

    
  virtual QString actionHandler (QString /*action*/, int /*session*/, int /*par1*/, int /*par2*/)
      { return QString::null; };
  virtual QString actionHandler (QString /*action*/, int /*session*/,
      const QString &/*par1*/, const QString &/*par2*/)
      { return QString::null; };
  virtual QString actionHandler (QString /*action*/, int /*session*/)
      { return QString::null; };
  virtual QString actionHandler (QString /*action*/, int /*session*/, cTextChunk * /*par*/)
      { return QString::null; };

  virtual void eventHandler (QString /*event*/, int /*session*/, int /*par1*/, int /*par2*/) {};
  virtual void eventHandler (QString /*event*/, int /*session*/,
      QString &/*par1*/, const QString &/*par2*/) {};
  virtual void eventHandler (QString /*event*/, int /*session*/) {};
  virtual void eventHandler (QString /*event*/, int /*session*/, cTextChunk * /*par*/) {};

  QString _name;
  cActionBasePrivate *d;

  // cActionManager will want to access event handlers ...
  friend class cActionManager;
};

#endif
