//
// C++ Interface: cActionManager
//
// Description: action manager
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef CACTIONMANAGER_H
#define CACTIONMANAGER_H

class cTextChunk;
struct cActionManagerPrivate;

class QWidget;
class KActionCollection;

#include "cactionbase.h"

/**
This is the action manager, it registers objects, actions and events, everyone who wants to use it must register with it. Registration is done automatically in cActionBase.


@author Tomas Mecir
*/
class cActionManager {
 public:
  static cActionManager *self ();
  /** destructor */
  ~cActionManager ();
  
  void registerSession (int id);
  void unregisterSession (int id);
  bool sessionExists (int id);
  
  void setSessionAttrib (int sess, const QString &name, int value);
  void setSessionStrAttrib (int sess, const QString &name, const QString &value);
  int sessionAttrib (int sess, const QString &name);
  QString sessionStrAttrib (int sess, const QString &name);
  
  int activeSession ();
  void setActiveSession (int sess);

  void registerObject (cActionBase *ab, int session = -1);
  void unregisterObject (cActionBase *ab, int session = -1);

  cActionBase *object (const QString &name, int session = 0);  

  /** add action of an object - name must be unique (per object) */
  void addEventHandler (cActionBase *ab, int session, QString name, int priority, ParamType pt);
  void removeEventHandler (cActionBase *ab, int session, QString name);
  
  /** call an object's action - PT_INT parameter */
  QString callAction (QString objectName, QString action, int session, int par1, int par2=0);
  /** call an object's action - PT_STRING parameter */
  QString callAction (QString objectName, QString action, int session,
      QString &par1, const QString &par2=QString::null);
  /** as above, but with const string */
  QString callAction (QString objectName, QString action, int session,
      const QString &par1, const QString &par2=QString::null);
  /** call an object's action - PT_NOTHING parameter */
  QString callAction (QString objectName, QString action, int session);
  /** call an object's action - PT_TEXTCHUNK parameter */
  QString callAction (QString objectName, QString action, int session, cTextChunk *par);
  
  /** invoke an event - PT_INT parameter */
  void invokeEvent (QString event, int session, int par1, int par2 = 0);
  /** invoke an event - PT_STRING parameter */
  void invokeEvent (QString event, int session, QString &par1,
      const QString &par2 = QString::null);
  /** as above, but const string */
  void invokeEvent (QString event, int session, const QString &par1,
      const QString &par2 = QString::null);
  /** invoke an event - PT_NOTHING parameter */
  void invokeEvent (QString event, int session);
  /** invoke an event - PT_TEXTCHUNK parameter */
  void invokeEvent (QString event, int session, cTextChunk *par);

  // Shortcuts - see comment below
  void createACol (QWidget *widget);
  KActionCollection *getACol ();
 private:
  /** constructor */
  cActionManager ();
  static cActionManager *_self;
  cActionManagerPrivate *d;
  
  // Shortcuts - only here because I have no better place for them :D
  // and I need them in lib/
  KActionCollection *acol;
};

#endif
