//
// C++ Interface: cMenuManager
//
// Description: Menu manager.
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef CMENUMANAGER_H
#define CMENUMANAGER_H

#include <cactionbase.h>

struct cMenuManagerPrivate;

class QPopupMenu;
class KAction;
class KMenuBar;

/**
Menu manager. It manages the menubar, allowing plug-ins to insert items into the menu.

@author Tomas Mecir
*/
class cMenuManager : public cActionBase
{
public:
  static cMenuManager *self ();
  ~cMenuManager ();

  /** assign a menubar to the object. It can't work without having one. */
  void setMenuBar (KMenuBar *menuBar);

  /** create a position for menus */
  void addMenuPosition (const QString &name);
  /** create a position for menu items */
  void addItemPosition (const QString &name, QPopupMenu *menu);
  
  /** add a new menu */
  void addMenu (QPopupMenu *menu, const QString &label, const QString &position, int id=-1);
  /** remove an existing menu */
  void removeMenu (QPopupMenu *menu);
  
  /** plug an action to a given position */
  void plug (KAction *action, QString position);
  /** unplug an action */
  void unplug (KAction *action);

protected:
  cMenuManager ();
  
  /** position of the given action */
  int positionOf (KAction *action);
  /** position of the given menubar */
  int positionOf (QPopupMenu *menu);
  /** menu assigned to action */
  QPopupMenu *menuOf (KAction *action);
  
  /** shift all existing entries following this position by a given delta */
  void shiftItems (const QString &position, int delta = 1);

  static cMenuManager *_self;
  
  cMenuManagerPrivate *d;
};

#endif
